package com.zenjava.resource;

import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * A default implementation of a ResourceManager that uses a standard Java ResourceBundle for all resources.
 */
public class DefaultResourceManager extends AbstractResourceManager
{
    private static final Logger logger = Logger.getLogger(DefaultResourceManager.class.getName());

    private String resourceBundleName;
    private Locale locale;
    private ResourceBundle resourceBundle;

    /**
     * Constructs a new instance of this class using the default resource bundle name of 'resources' and the current
     * default Locale.
     */
    public DefaultResourceManager()
    {
        this("resources");
    }

    /**
     * Constructs a new instance of this class using the resource bundle name specified and the current default Locale.
     *
     * @param resourceBundleName the name of the resource bundle to use. This will be the base name of the resource
     * properties files loaded based on the current Locale.
     */
    public DefaultResourceManager(String resourceBundleName)
    {
        this(resourceBundleName, Locale.getDefault());
    }

    /**
     * Constructs a new instance of this class using the resource bundle name and Locale specified.
     *
     * @param resourceBundleName the name of the resource bundle to use. This will be the base name of the resource
     * properties files loaded based on the current Locale.
     * @param locale the Locale to load resources for.
     */
    public DefaultResourceManager(String resourceBundleName, Locale locale)
    {
        if (logger.isLoggable(Level.INFO))
        {
            logger.info(String.format("Initialising DefaultResourceManager with resource bundle '%s' and locale '%s'",
                    resourceBundleName, locale));
        }
        this.resourceBundleName = resourceBundleName;
        setLocale(locale);
    }

    /**
     * Sets the current Locale of this ResourceManager to the one specified. This will cause the current ResourceBundle
     * to be discarded and a new one to be loaded based on this new Locale and the resource bundle name that this
     * ResourceManager was created with.
     *
     * @param locale the new locale to use. This must not be null.
     */
    public void setLocale(Locale locale)
    {
        if (logger.isLoggable(Level.INFO))
        {
            logger.info(String.format("Setting locale to %s", locale.getDisplayName()));
        }

        this.locale = locale;

        try
        {
            resourceBundle = ResourceBundle.getBundle(resourceBundleName, locale);
        }
        catch (MissingResourceException e)
        {
            resourceBundle = null;
            if (logger.isLoggable(Level.INFO))
            {
                logger.info(String.format("No resource bundle found for '%s', relying on default annotation values", resourceBundleName));
            }
        }
        fireResourcesUpdated(new ResourceManagerEvent(this));
    }

    /**
     * Retrieves the current Locale that this ResourceManager is configured to use.
     *
     * @return the current Locale for this ResourceManager.
     */
    public Locale getLocale()
    {
        return locale;
    }

    /**
     * Retrieves the localized text resource for the specified lookup key using the Locale that this ResourceManager has
     * been configured to use. This will lookup the value from the current ResourceBundle using the specified key
     * directly as the lookup key.
     *
     * @param key the lookup key for the value to be localized.
     * @return the value for the specified lookup key based on the Locale this ResourceManager is configured to use. If
     * the configured ResourceBundle does not contain a value for the specified key then null is returned.
     */
    public String getString(String key)
    {
        try
        {
            String value = resourceBundle != null ? resourceBundle.getString(key) : null;
            if (logger.isLoggable(Level.FINEST))
            {
                logger.finest(String.format("Localized value for '%s' is '%s' (locale='%s')", key, value, locale.getDisplayName()));
            }
            return value;
        }
        catch (MissingResourceException e)
        {
            if (logger.isLoggable(Level.FINE))
            {
                logger.finest(String.format("Localized value for '%s' is not defined for locale '%s'", key, locale.getDisplayName()));
            }
            return null;
        }
    }
}
