package com.zenjava.resource;

import java.lang.reflect.Proxy;

/**
 * Factory for creating instances of resource providers from interfaces. Instances are bound to an underlying
 * ResourceManager, which is the source of all the resources that the resource providers provide simplified, type-safe
 * access to.
 */
public class ResourceProviderFactory
{
    private ResourceManager resourceManager;

    /**
     * Constructs a new instance of this class with no underlying ResourceManager. A ResourceManager must be set before
     * this factory can be used.
     */
    public ResourceProviderFactory()
    {
    }

    /**
     * Constructs a new instance of this class, using the specified ResourceManager as the source for all localized
     * resources.
     *
     * @param resourceManager the ResourceManager to use as the source for all localized resources.
     */
    public ResourceProviderFactory(ResourceManager resourceManager)
    {
        this.resourceManager = resourceManager;
    }

    /**
     * Sets the ResourceManager to use as the source for all localized resources. This will only affect resource
     * providers that are subsequently created, it will not affect any resource providers that were created previously,
     * these will continue to use the previous ResourceManager.
     *
     * @param resourceManager the ResourceManager to use as the source for all localized resources.
     */
    public void setResourceManager(ResourceManager resourceManager)
    {
        this.resourceManager = resourceManager;
    }

    /**
     * Creates a new resource provider instance based on the specified resource interface. The resulting resource
     * provider will use the ResourceProviderProxy to dynamically bind the methods of the resource interface to the
     * ResourceManager of this factory. The resulting instance will honour the Zen Resources 'Localized' annotations.
     *
     * @param resourceInterface the interface defining the resources that the provider is to provide localized access to.
     * @param <T> the type of the interface specified. The return type will be an instance of the type provided.
     * @return an instance of a resource provider that binds the methods of the resource interface to the underlying
     * ResourceManager.
     */
    @SuppressWarnings("unchecked")
    public <T> T getResourceProvider(Class<T> resourceInterface)
    {
        return (T)Proxy.newProxyInstance(
                resourceInterface.getClassLoader(),
                new Class[]{resourceInterface},
                new ResourceProviderProxy(resourceManager));
    }
}
