package com.zenjava.resource;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;

/**
 * Provides a proxy for binding resource interfaces to a ResourceManager. When a method on the resource interface is
 * invoked, this proxy uses reflection to determine the lookup key (either via the class and method name, or via the
 * annotations specified on the method) to determine the lookup key to use with the ResourceManager.
 */
public class ResourceProviderProxy implements InvocationHandler
{
    private ResourceManager resourceManager;

    /**
     * Constructs a new instance of this class using the specified ResourceManager as the source for localized resource
     * values.
     *
     * @param resourceManager the source for localized resource values for this proxy.
     */
    public ResourceProviderProxy(ResourceManager resourceManager)
    {
        this.resourceManager = resourceManager;
    }

    /**
     * Handles the invocation of a method on the resource interface being proxied. The resulting action depends on the
     * method being invoked, the annotations on that method and the underlying ResourceManager.
     * <p/>
     * If the specified method returns a String and takes no parameters, then the full class name (including package
     * name) of the declaring interface of the method will be prepended to the method name to determine the lookup key
     * in the underlying ResourceManager.
     * <p/>
     * If the specified method returns a String and has one or more parameters (and is not annotated with
     * LocalizedEnum), then the localized value is passed to the String.format() method, with the method arguments
     * passed as substitutions.
     * <p/>
     * If the return type of the method is a DateFormat then this handler will create and return a SimpleDateFormat
     * instance using the localized text value from the ResourceManager as the format. Similarly, if the return type is
     * a NumberFormat, then this handler will create and return a DecimalFormat using the localized text value as the
     * format.
     * <p/>
     * This method will honour the Localized, LocalizedEnum and LocalizedEnumValue annotations. See the JavaDoc on these
     * annotations for more information.
     *
     * @param proxy  the instance of the proxy object that the method was invoked on.
     * @param method the method that was invoked.
     * @param args   the arguments that were used when the method was invoked.
     * @return the result of this handler implementing the method invocation on behalf of the proxy.
     * @throws Throwable any error that occurs while handling the method.
     */
    public Object invoke(Object proxy, Method method, Object[] args) throws Throwable
    {
        if (method.getDeclaringClass().equals(ResourceEventSource.class))
        {
            return method.invoke(resourceManager, args);
        }

        LocalizedEnum localizedEnumAnnotation = method.getAnnotation(LocalizedEnum.class);
        if (localizedEnumAnnotation != null)
        {
            if (args != null && args.length == 1 && args[0] instanceof Enum)
            {
                String name = ((Enum) args[0]).name();

                // determine base key from annotation or from method name if not defined on the annotation
                String baseKey = localizedEnumAnnotation.key();
                if (baseKey == null || baseKey.equals(""))
                {
                    baseKey = getKey(method);
                }

                String key = String.format("%s.%s", baseKey, name);
                String value = resourceManager.getString(key);

                // use annotation defaults if no value defined by resource manager
                if (value == null)
                {
                    if (localizedEnumAnnotation.mappings() != null)
                    {
                        for (LocalizedEnumValue localizedEnumValueAnnotation : localizedEnumAnnotation.mappings())
                        {
                            if (name.equals(localizedEnumValueAnnotation.enumKey()))
                            {
                                value = localizedEnumValueAnnotation.value();
                                break;
                            }
                        }
                    }
                }

                return value;
            }
            else
            {
                throw new InvalidResourceMappingException(String.format(
                        "Resource method '%s' annotated with @MappedEnum must have exactly one, non-null enum parameter",
                        method.getName()));
            }
        }
        else
        {
            String key = getKey(method);
            String value = getString(key, args, method);
            if (method.getReturnType().equals(String.class))
            {
                return value;
            }
            else if (method.getReturnType().equals(DateFormat.class))
            {
                // todo this value could be cached
                return value != null ? new SimpleDateFormat(value) : null;
            }
            else if (method.getReturnType().equals(NumberFormat.class))
            {
                // todo this value could be cached
                return value != null ? new DecimalFormat(value) : null;
            }
            else
            {
                throw new InvalidResourceMappingException(String.format(
                        "Method '%s' has an unsupported return type '%s' for resource mapping",
                        method.getName(), method.getReturnType()));
            }
        }
    }

    /**
     * Determine the lookup key for the specified method. The key will be the full name of the Method's declaring class
     * (including package) prepended to the method name. This can be overridden by the LocalizedValue annotation however.
     *
     * @param method the method to determine the lookup key for.
     * @return the lookup key based on the method  provided.
     */
    protected String getKey(Method method)
    {
        LocalizedValue annotation = method.getAnnotation(LocalizedValue.class);

        String key;

        if (annotation != null && annotation.key() != null && !annotation.key().equals(""))
        {
            key = annotation.key();
        }
        else
        {
            key = String.format("%s.%s", method.getDeclaringClass().getName(), method.getName());
        }

        return key;
    }

    /**
     * Gets the localized value for the specified key and method invocation. The key is used to lookup the value in the
     * ResourceManager. If no value is found, then null is returned, unless the method is annotated with the
     * LocalizedValue annotation, specifying an alternate default value. If the resulting value is not null and the
     * method was invoked with arguments, then the value is passed to the String.format() method, passing the method
     * arguments as substitution parameters.
     *
     * @param key    the key to lookup the value for.
     * @param args   the arguments used to invoke the localized method.
     * @param method the method that was invoked to determine the localized value.
     * @return the resulting localized value for the invocation.
     */
    protected String getString(String key, Object[] args, Method method)
    {
        String value = resourceManager.getString(key);

        // use annotation default value if no value was found in the resource bundle
        if (value == null)
        {
            LocalizedValue annotation = method.getAnnotation(LocalizedValue.class);
            if (annotation != null && annotation.value() != null && !annotation.value().equals(""))
            {
                value = annotation.value();
            }
        }

        // apply substitutions
        if (value != null && args != null && args.length > 0 && !(args.length == 1 && args[0] != null && args[0] instanceof Enum))
        {
            value = String.format(value, args);
        }

        return value;
    }
}
